<?php
	/**
	 *	HTTP_Server class
	 *	This small class provides easy access to web server responses and client requests
	 *	This class should be primary used with WebDAV (it is it's main purpose)
	 *	Class support 1.0 and 1.1 version of HTTP protocol, with restriction, that 1.1 version uses <strong>Connection</strong> header.
	 *	It means that, connection is closed after each request with <strong>Connection: close</strong> header.
	 *
	 *	@package	webdav
	 *	@version	1.0 - public alpha
	 *	@author		Stepan Kouba <stepankouba@formiginal.com>
	 */
	class HTTP_Server
	{
		/**
		 *	host name
		 *	@var	string
		 */
		var $host;
		
		/**
		 *	port number
		 *	@access	private
		 *	@var	int
		 */
		var $port;
		
		/**
		 *	version of used HTTP protocol
		 *	@var	string
		 */
		var $httpVersion;
		
		/**
		 *	number of socket
		 *	@access	private
		 *	@var	int
		 */
		var $socket;
		
		/**
		 *	error string
		 *	@var	string
		 *	@access	private
		 */
		var $errStr;
		
		/**
		 *	error number
		 *	@access	private
		 *	@var	int
		 */
		var $errNo;
		
		/**
		 *	client's request body (XML document)
		 *	@access private
		 *	@var	mixed
		 */
		var $clientBody = false;
		
		/**
		 *	whole server request containing HTTP headers, XML body,...
		 *	@access private
		 *	@var	mixed
		 */
		var $serverResponse = false;
		
		/**
		 *	server's response body (XML document, HTML document, pictures,...)
		 *	@access private
		 *	@var	mixed
		 */
		var $serverBody = false;
		
		/**
		 *	server's response headers
		 *	@access	private
		 *	@var	mixed
		 */
		var $serverHeaders = false;
		
		/**
		 *	MIME type of client body (usually text/xml or application/xml)
		 *	@access	private
		 *	@var	string
		 */
		var $clientBodyType;
		
		/**
		 *	headers which user want to send with command. Headers are saved as associative array ("Depth" => "0")
		 *	@access	private
		 *	@var	array
		 */
		var $headers;
		
		/**
		 *	set variables $host, $port and $httpVersion
		 *	
		 *	@param	string	$host	host site
		 *	@param	int		$port	port number
		 *	@param	string	$httpVersion	number of used HTTP version
		 */
		function HTTP_Server( $host, $port = 80, $httpVersion = "1.1" )
		{
			$this->host = $host;
			$this->port = $port;
			$this->httpVersion = $httpVersion;
		}// constructor
		
		/**
		 *	connect http server and open socket
		 *	@return	bool
		 */
		function connect()
		{
			$this->socket = fsockOpen( $this->host, $this->port, $this->errNo, $this->errStr, 30 );
			
			if ( !$this->socket )
				die( $this->errNo.": ".$this->errStr );
			
			return true;
		}// function
		
		/**
		 *	close connection
		 *	@return	bool
		 */
		function close()
		{
			fclose( $this->socket );
			
			return true;
		}// function
		
		/**
		 *	send client request. If request has it's body, it must be set via setClientBody method
		 *	@param	string	$command	HTTP command
		 *	@param	string	$file		file (directory) which will be with command affected
		 */
		function request( $command, $file )
		{
			$request = $command." ".$file." HTTP/".$this->httpVersion."\r\n";
			
			// if we use HTTP/1.1, then is neccessary to send Host header
			if ( $this->httpVersion == "1.1" )
				$request .= "Host: ".$this->host."\r\n";
			
			if ( $this->clientBody )
			{
				$request .= "Content-type: ".$this->clientBodyType."; charset=\"iso-8859-1\"\r\n";
				$request .= "Content-length: ".strLen( $this->clientBody )."\r\n";
			}// if
			
			// create headers
			if ( is_array( $this->headers ) )
			{
				foreach ( $this->headers as $name => $value )
				{
					$request .= $name.": ".$value."\r\n";
				}// foreach
			}// function
			
			// enforce closing connection if we use version 1.1
			if ( $this->httpVersion == "1.1" )
				$request .= "Connection: close\r\n";
			
			// finish creating request header
			$request .= "\r\n";
			
			if ( $this->clientBody )
				$request .= $this->clientBody;
			
			// send request
			fPuts( $this->socket, $request );
		}// function
		
		/**
		 *	set part of client request (usually XML document)
		 *	@param	string	$body	entire body
		 *	@param	strinf	$type	MIME type of body content
		 */
		function setClientBody( $body, $type )
		{
			$this->clientBody = $body;
			$this->clientBodyType = $type;
		}// function
		
		/**
		 *	get server's response. After calling this method property serverBody will be set.
		 */
		function response()
		{
			$response = "";
			
			// read response until end
			while ( !fEof( $this->socket ) )
				$response .= fGetS( $this->socket, 256 );
			
			// save server's response
			$this->serverResponse = $response;
			
			// save server's body
			list( $this->serverHeaders, $this->serverBody ) = $this->parseResponse( $response );
			
			// clear headers array (must be empty for other request)
			$this->headers = array();
			
			// clear client body for other request
			$this->clientBody = false;
		}// function
		
		/**
		 *	get entire server's response (HTTP headers, body, ...)
		 *	@return string
		 */
		function getServerResponse()
		{
			return $this->serverResponse;
		}// function
		
		/**
		 *	get body of server's response.
		 *	@return string
		 */
		function getServerBody()
		{
			return $this->serverBody;
		}// function
		
		/**
		 *	get headers returned by server.
		 *	@return string
		 */
		function getServerHeaders()
		{
			return $this->serverHeaders;
		}// function
		
		/**
		 *	use this function to set all neccessary headers
		 *	@param	string	$header	header name
		 *	@param	string	$value	header's value
		 */
		function setHeader( $header, $value )
		{
			if ( !is_array( $this->headers ) )
				$this->headers = array();
			
			$this->headers[ucFirst( strToLower( $header ) )] = $value;
		}// function
		
		/**
		 *	get server's response and split it into two parts (headers and body).
		 *	If we use HTTP/1.0 then return array containing both parts, if we use higher version, then we have to analyze headers
		 *	(we are looking for header <strong>Content-length</strong> or chunk start).
		 *	Array structure:	[0] => headers
		 *						[1] => body
		 *	
		 *	@access	private
		 *	@param	string	$data	server's response
		 *	@return array	server's headers and body
		 */
		function parseResponse( $data )
		{
			// get body from response
			$a = explode( "\r\n\r\n", $data );
			
			// for version 1.0 return whole body
			if ( $this->httpVersion == "1.0" )
			{
				return $a;
			}// if
			// for version 1.1 return body
			elseif ( $this->httpVersion == "1.1" )
			{
				// get length from header chunk or from Content-Length
				if ( $this->headerValue( "Transfer-Encoding", $a[0] ) == "chunked" )
				{
					$a[1] = $this->joinChunked( $a[1] );
				}// if
				else
				{
					$strLength = $this->headerValue( "Content-Length", $a[0] );
					$a[1] = subStr( $a[1], 0, $strLength );
				}// else
				
				// return array
				return $a;
			}// elseif
		}// function
		
		/**
		 *	get body which is divided into chunkes and imlode it together. This method is neccessary for supporting version 1.1 of
		 *	HTTP protocol. 
		 *	
		 *	@access	private
		 *	@param	string	$body	response's body
		 *	@return string	modified body
		 */
		function joinChunked( $body )
		{
            $chunks = "";
			
            do
			{
                // set length
				$length = 0;
				
				// read first line (it contains hexadecimal number representing length of chunk)
                $line = strTok( $body, "\r\n" );
                
				// get body without the first line
				$body = subStr( $body, strLen( $line ) + 2 );
				
				// if first line has hexdec number...
                if ( ctype_xdigit( $line ) )
				{
					// get length
                    $length = hexDec( $line );
					
					// get chunk and delete it from body
                    if ( $length > 0 )
					{
                        $chunks .= subStr( $body, 0, $length );
                        $body = subStr( $body, $length + 2 );
                    }// if
                }// if
			} while ( $length != 0 );
			
			return $chunks;
		}// function
		
		/**
		 *	returns status code in array, which has the following structure:
		 *		["line"] => whole status line (HTTP/1.1 204 OK)
		 *		["code"] => status code (204)
		 *		["phrase"] => response phrase (OK)
		 *		["version"] => HTTP version (1.1)
		 *	
		 *	@return array	status cod, phrase and whole line (see description for more)
		 */
		function getStatus()
		{
			// get first line of response (status line)
			$statusLine = strTok( $this->serverResponse, "\r\n" );
			
			// explode it with space
			ereg( "^HTTP/(.{3}) ([0-9]{3}) (.*)",  $statusLine, $a );
			
			return array( "line" => $statusLine, "code" => $a[2], "phrase" => $a[3], "version" => $a[1] );
		}// function
		
		/**
		 *	returns value of header which is specified with the first parameter. Second (optional) parameter may contain
		 *	string (usualy headers :-]) where value shoul be found. If not set, then {@link HTTP_Server::serverHeaders} will be
		 *	searched.
		 *	
		 *	@param	string	$header		header name (needle)
		 *	@param	mixec	$string		haystack
		 */
		function headerValue( $header, $string =  false )
		{
			$hs = ( $string ) ? $string : $this->serverHeaders;
			
			if ( ereg( $header.": ([[:graph:]]*)", $hs, $a ) )
				return $a[1];
			else
				return false;
		}// function
	}// class
?>